package dev.fyloz.colorrecipesexplorer.utils

import dev.fyloz.colorrecipesexplorer.Constants
import javax.servlet.http.HttpServletResponse


data class CookieBuilderOptions(
    /** HTTP Only cookies cannot be access by Javascript clients. */
    var httpOnly: Boolean = DEFAULT_HTTP_ONLY,

    /** SameSite cookies are only sent in requests to their origin location. */
    var sameSite: Boolean = DEFAULT_SAME_SITE,

    /** Secure cookies are only sent in HTTPS requests. */
    var secure: Boolean = DEFAULT_SECURE,

    /** Cookie's maximum age in seconds. */
    var maxAge: Long? = DEFAULT_MAX_AGE,

    /** The path for which the cookie will be sent. */
    var path: String = DEFAULT_PATH
) {
    companion object {
        private const val DEFAULT_MAX_AGE = 3600L
        private const val DEFAULT_HTTP_ONLY = true
        private const val DEFAULT_SAME_SITE = true
        private const val DEFAULT_SECURE = true
        private const val DEFAULT_PATH = Constants.ControllerPaths.BASE_PATH
    }
}

private enum class CookieBuilderOption(val optionName: String) {
    HTTP_ONLY("HttpOnly"),
    SAME_SITE("SameSite"),
    SECURE("Secure"),
    MAX_AGE("Max-Age"),
    PATH("Path")
}

fun HttpServletResponse.addCookie(name: String, value: String, optionsBuilder: CookieBuilderOptions.() -> Unit) {
    this.addHeader("Set-Cookie", buildCookie(name, value, optionsBuilder))
}

private fun buildCookie(name: String, value: String, optionsBuilder: CookieBuilderOptions.() -> Unit): String {
    val options = CookieBuilderOptions().apply(optionsBuilder)
    val cookie = StringBuilder("$name=$value;")

    fun addBoolOption(option: CookieBuilderOption, enabled: Boolean) {
        if (enabled) {
            cookie.append("${option.optionName};")
        }
    }

    fun addOption(option: CookieBuilderOption, value: Any?) {
        if (value == null) return
        cookie.append("${option.optionName}=$value;")
    }

    addBoolOption(CookieBuilderOption.HTTP_ONLY, options.httpOnly)
    addBoolOption(CookieBuilderOption.SAME_SITE, options.sameSite)
    addBoolOption(CookieBuilderOption.SECURE, options.secure)
    addOption(CookieBuilderOption.MAX_AGE, options.maxAge)
    addOption(CookieBuilderOption.PATH, options.path)

    return cookie.toString()
}

fun parseBearer(source: String) =
    source.replace(Constants.BEARER_PREFIX, "").trim()